@@ -36,12 +36,13 @@ import org.apache.commons.math3.fraction.BigFraction;
 import org.apache.commons.math3.fraction.Fraction;
 import org.apache.commons.math3.util.FastMath;
 import org.apache.commons.math3.util.MathArrays;
+import org.apache.commons.math3.util.MathUtils;
 import org.apache.commons.math3.util.Precision;
 
 /**
  * A collection of static methods that operate on or return matrices.
  *
- * @version $Id: MatrixUtils.java 1449528 2013-02-24 19:06:20Z luc $
+ * @version $Id: MatrixUtils.java 1533638 2013-10-18 21:19:18Z tn $
  */
 public class MatrixUtils {
 
@@ -1023,18 +1024,34 @@ public class MatrixUtils {
         final RealMatrix d = m.getSubMatrix(splitIndex1, n - 1, splitIndex1, n - 1);
 
         final SingularValueDecomposition aDec = new SingularValueDecomposition(a);
-        final RealMatrix aInv = aDec.getSolver().getInverse();
+        final DecompositionSolver aSolver = aDec.getSolver();
+        if (!aSolver.isNonSingular()) {
+            throw new SingularMatrixException();
+        }
+        final RealMatrix aInv = aSolver.getInverse();
 
         final SingularValueDecomposition dDec = new SingularValueDecomposition(d);
-        final RealMatrix dInv = dDec.getSolver().getInverse();
+        final DecompositionSolver dSolver = dDec.getSolver();
+        if (!dSolver.isNonSingular()) {
+            throw new SingularMatrixException();
+        }
+        final RealMatrix dInv = dSolver.getInverse();
 
         final RealMatrix tmp1 = a.subtract(b.multiply(dInv).multiply(c));
         final SingularValueDecomposition tmp1Dec = new SingularValueDecomposition(tmp1);
-        final RealMatrix result00 = tmp1Dec.getSolver().getInverse();
+        final DecompositionSolver tmp1Solver = tmp1Dec.getSolver();
+        if (!tmp1Solver.isNonSingular()) {
+            throw new SingularMatrixException();
+        }
+        final RealMatrix result00 = tmp1Solver.getInverse();
 
         final RealMatrix tmp2 = d.subtract(c.multiply(aInv).multiply(b));
         final SingularValueDecomposition tmp2Dec = new SingularValueDecomposition(tmp2);
-        final RealMatrix result11 = tmp2Dec.getSolver().getInverse();
+        final DecompositionSolver tmp2Solver = tmp2Dec.getSolver();
+        if (!tmp2Solver.isNonSingular()) {
+            throw new SingularMatrixException();
+        }
+        final RealMatrix result11 = tmp2Solver.getInverse();
 
         final RealMatrix result01 = aInv.multiply(b).multiply(result11).scalarMultiply(-1);
         final RealMatrix result10 = dInv.multiply(c).multiply(result00).scalarMultiply(-1);
@@ -1047,4 +1064,57 @@ public class MatrixUtils {
 
         return result;
     }
+
+    /**
+     * Computes the inverse of the given matrix.
+     * <p>
+     * By default, the inverse of the matrix is computed using the QR-decomposition,
+     * unless a more efficient method can be determined for the input matrix.
+     * <p>
+     * Note: this method will use a singularity threshold of 0,
+     * use {@link #inverse(RealMatrix, double)} if a different threshold is needed.
+     *
+     * @param matrix Matrix whose inverse shall be computed
+     * @return the inverse of {@code matrix}
+     * @throws NullArgumentException if {@code matrix} is {@code null}
+     * @throws SingularMatrixException if m is singular
+     * @throws NonSquareMatrixException if matrix is not square
+     * @since 3.3
+     */
+    public static RealMatrix inverse(RealMatrix matrix)
+            throws NullArgumentException, SingularMatrixException, NonSquareMatrixException {
+        return inverse(matrix, 0);
+    }
+
+    /**
+     * Computes the inverse of the given matrix.
+     * <p>
+     * By default, the inverse of the matrix is computed using the QR-decomposition,
+     * unless a more efficient method can be determined for the input matrix.
+     *
+     * @param matrix Matrix whose inverse shall be computed
+     * @param threshold Singularity threshold
+     * @return the inverse of {@code m}
+     * @throws NullArgumentException if {@code matrix} is {@code null}
+     * @throws SingularMatrixException if matrix is singular
+     * @throws NonSquareMatrixException if matrix is not square
+     * @since 3.3
+     */
+    public static RealMatrix inverse(RealMatrix matrix, double threshold)
+            throws NullArgumentException, SingularMatrixException, NonSquareMatrixException {
+
+        MathUtils.checkNotNull(matrix);
+
+        if (!matrix.isSquare()) {
+            throw new NonSquareMatrixException(matrix.getRowDimension(),
+                                               matrix.getColumnDimension());
+        }
+
+        if (matrix instanceof DiagonalMatrix) {
+            return ((DiagonalMatrix) matrix).inverse(threshold);
+        } else {
+            QRDecomposition decomposition = new QRDecomposition(matrix, threshold);
+            return decomposition.getSolver().getInverse();
+        }
+    }
 }
