@@ -43,7 +43,7 @@ import org.apache.commons.math3.util.Precision;
 
 /**
  * Base class managing common boilerplate for all integrators.
- * @version $Id: AbstractIntegrator.java 1463684 2013-04-02 19:04:13Z luc $
+ * @version $Id: AbstractIntegrator.java 1517418 2013-08-26 03:18:55Z dbrosius $
  * @since 2.0
  */
 public abstract class AbstractIntegrator implements FirstOrderIntegrator {
@@ -141,7 +141,7 @@ public abstract class AbstractIntegrator implements FirstOrderIntegrator {
 
     /** {@inheritDoc} */
     public Collection<EventHandler> getEventHandlers() {
-        final List<EventHandler> list = new ArrayList<EventHandler>();
+        final List<EventHandler> list = new ArrayList<EventHandler>(eventsStates.size());
         for (EventState state : eventsStates) {
             list.add(state.getEventHandler());
         }
@@ -188,6 +188,7 @@ public abstract class AbstractIntegrator implements FirstOrderIntegrator {
         evaluations.resetCount();
 
         for (final EventState state : eventsStates) {
+            state.setExpandable(expandable);
             state.getEventHandler().init(t0, y0, t);
         }
 
@@ -326,7 +327,7 @@ public abstract class AbstractIntegrator implements FirstOrderIntegrator {
 
             // search for next events that may occur during the step
             final int orderingSign = interpolator.isForward() ? +1 : -1;
-            SortedSet<EventState> occuringEvents = new TreeSet<EventState>(new Comparator<EventState>() {
+            SortedSet<EventState> occurringEvents = new TreeSet<EventState>(new Comparator<EventState>() {
 
                 /** {@inheritDoc} */
                 public int compare(EventState es0, EventState es1) {
@@ -338,14 +339,14 @@ public abstract class AbstractIntegrator implements FirstOrderIntegrator {
             for (final EventState state : eventsStates) {
                 if (state.evaluateStep(interpolator)) {
                     // the event occurs during the current step
-                    occuringEvents.add(state);
+                    occurringEvents.add(state);
                 }
             }
 
-            while (!occuringEvents.isEmpty()) {
+            while (!occurringEvents.isEmpty()) {
 
                 // handle the chronologically first event
-                final Iterator<EventState> iterator = occuringEvents.iterator();
+                final Iterator<EventState> iterator = occurringEvents.iterator();
                 final EventState currentEvent = iterator.next();
                 iterator.remove();
 
@@ -356,7 +357,6 @@ public abstract class AbstractIntegrator implements FirstOrderIntegrator {
 
                 // get state at event time
                 interpolator.setInterpolatedTime(eventT);
-                final double[] eventYPrimary  = interpolator.getInterpolatedState().clone();
                 final double[] eventYComplete = new double[y.length];
                 expandable.getPrimaryMapper().insertEquationData(interpolator.getInterpolatedState(),
                                                                  eventYComplete);
@@ -368,7 +368,7 @@ public abstract class AbstractIntegrator implements FirstOrderIntegrator {
 
                 // advance all event states to current time
                 for (final EventState state : eventsStates) {
-                    state.stepAccepted(eventT, eventYPrimary);
+                    state.stepAccepted(eventT, eventYComplete);
                     isLastStep = isLastStep || state.stop();
                 }
 
@@ -405,14 +405,21 @@ public abstract class AbstractIntegrator implements FirstOrderIntegrator {
                 // check if the same event occurs again in the remaining part of the step
                 if (currentEvent.evaluateStep(interpolator)) {
                     // the event occurs during the current step
-                    occuringEvents.add(currentEvent);
+                    occurringEvents.add(currentEvent);
                 }
 
             }
 
             // last part of the step, after the last event
             interpolator.setInterpolatedTime(currentT);
-            final double[] currentY = interpolator.getInterpolatedState();
+            final double[] currentY = new double[y.length];
+            expandable.getPrimaryMapper().insertEquationData(interpolator.getInterpolatedState(),
+                                                             currentY);
+            int index = 0;
+            for (EquationsMapper secondary : expandable.getSecondaryMappers()) {
+                secondary.insertEquationData(interpolator.getInterpolatedSecondaryState(index++),
+                                             currentY);
+            }
             for (final EventState state : eventsStates) {
                 state.stepAccepted(currentT, currentY);
                 isLastStep = isLastStep || state.stop();
