@@ -21,6 +21,7 @@ import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.io.Serializable;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.HashSet;
 import java.util.List;
@@ -29,11 +30,9 @@ import java.util.TreeSet;
 
 import org.apache.commons.math3.linear.Array2DRowRealMatrix;
 import org.apache.commons.math3.linear.MatrixUtils;
-import org.apache.commons.math3.linear.RealMatrix;
 import org.apache.commons.math3.linear.RealVector;
 import org.apache.commons.math3.optim.nonlinear.scalar.GoalType;
 import org.apache.commons.math3.optim.PointValuePair;
-import org.apache.commons.math3.util.FastMath;
 import org.apache.commons.math3.util.Precision;
 
 /**
@@ -58,7 +57,7 @@ import org.apache.commons.math3.util.Precision;
  * a1: Artificial variable</br>
  * RHS: Right hand side</br>
  * </p>
- * @version $Id: SimplexTableau.java 1435810 2013-01-20 10:04:45Z tn $
+ * @version $Id: SimplexTableau.java 1554544 2013-12-31 15:40:35Z tn $
  * @since 2.0
  */
 class SimplexTableau implements Serializable {
@@ -82,7 +81,7 @@ class SimplexTableau implements Serializable {
     private final List<String> columnLabels = new ArrayList<String>();
 
     /** Simple tableau. */
-    private transient RealMatrix tableau;
+    private transient Array2DRowRealMatrix tableau;
 
     /** Number of decision variables. */
     private final int numDecisionVariables;
@@ -99,8 +98,11 @@ class SimplexTableau implements Serializable {
     /** Amount of error to accept in floating point comparisons. */
     private final int maxUlps;
 
-    /** Cut-off value for entries in the tableau. */
-    private final double cutOff;
+    /** Maps basic variables to row they are basic in. */
+    private int[] basicVariables;
+
+    /** Maps rows to their corresponding basic variables. */
+    private int[] basicRows;
 
     /**
      * Builds a tableau for a linear problem.
@@ -117,8 +119,7 @@ class SimplexTableau implements Serializable {
                    final GoalType goalType,
                    final boolean restrictToNonNegative,
                    final double epsilon) {
-        this(f, constraints, goalType, restrictToNonNegative, epsilon,
-                SimplexSolver.DEFAULT_ULPS, SimplexSolver.DEFAULT_CUT_OFF);
+        this(f, constraints, goalType, restrictToNonNegative, epsilon, SimplexSolver.DEFAULT_ULPS);
     }
 
     /**
@@ -136,39 +137,20 @@ class SimplexTableau implements Serializable {
                    final boolean restrictToNonNegative,
                    final double epsilon,
                    final int maxUlps) {
-        this(f, constraints, goalType, restrictToNonNegative, epsilon, maxUlps, SimplexSolver.DEFAULT_CUT_OFF);
-    }
-
-    /**
-     * Build a tableau for a linear problem.
-     * @param f linear objective function
-     * @param constraints linear constraints
-     * @param goalType type of optimization goal: either {@link GoalType#MAXIMIZE} or {@link GoalType#MINIMIZE}
-     * @param restrictToNonNegative whether to restrict the variables to non-negative values
-     * @param epsilon amount of error to accept when checking for optimality
-     * @param maxUlps amount of error to accept in floating point comparisons
-     * @param cutOff the cut-off value for tableau entries
-     */
-    SimplexTableau(final LinearObjectiveFunction f,
-                   final Collection<LinearConstraint> constraints,
-                   final GoalType goalType,
-                   final boolean restrictToNonNegative,
-                   final double epsilon,
-                   final int maxUlps,
-                   final double cutOff) {
         this.f                      = f;
         this.constraints            = normalizeConstraints(constraints);
         this.restrictToNonNegative  = restrictToNonNegative;
         this.epsilon                = epsilon;
         this.maxUlps                = maxUlps;
-        this.cutOff                 = cutOff;
-        this.numDecisionVariables   = f.getCoefficients().getDimension() +
-                                      (restrictToNonNegative ? 0 : 1);
+        this.numDecisionVariables   = f.getCoefficients().getDimension() + (restrictToNonNegative ? 0 : 1);
         this.numSlackVariables      = getConstraintTypeCounts(Relationship.LEQ) +
                                       getConstraintTypeCounts(Relationship.GEQ);
         this.numArtificialVariables = getConstraintTypeCounts(Relationship.EQ) +
                                       getConstraintTypeCounts(Relationship.GEQ);
         this.tableau = createTableau(goalType == GoalType.MAXIMIZE);
+        // initialize the basic variables for phase 1:
+        //   we know that only slack or artificial variables can be basic
+        initializeBasicVariables(getSlackVariableOffset());
         initializeColumnLabels();
     }
 
@@ -200,7 +182,7 @@ class SimplexTableau implements Serializable {
      * @param maximize if true, goal is to maximize the objective function
      * @return created tableau
      */
-    protected RealMatrix createTableau(final boolean maximize) {
+    protected Array2DRowRealMatrix createTableau(final boolean maximize) {
 
         // create a matrix of the correct size
         int width = numDecisionVariables + numSlackVariables +
@@ -212,17 +194,16 @@ class SimplexTableau implements Serializable {
         if (getNumObjectiveFunctions() == 2) {
             matrix.setEntry(0, 0, -1);
         }
+
         int zIndex = (getNumObjectiveFunctions() == 1) ? 0 : 1;
         matrix.setEntry(zIndex, zIndex, maximize ? 1 : -1);
-        RealVector objectiveCoefficients =
-            maximize ? f.getCoefficients().mapMultiply(-1) : f.getCoefficients();
+        RealVector objectiveCoefficients = maximize ? f.getCoefficients().mapMultiply(-1) : f.getCoefficients();
         copyArray(objectiveCoefficients.toArray(), matrix.getDataRef()[zIndex]);
-        matrix.setEntry(zIndex, width - 1,
-            maximize ? f.getConstantTerm() : -1 * f.getConstantTerm());
+        matrix.setEntry(zIndex, width - 1, maximize ? f.getConstantTerm() : -1 * f.getConstantTerm());
 
         if (!restrictToNonNegative) {
             matrix.setEntry(zIndex, getSlackVariableOffset() - 1,
-                getInvertedCoefficientSum(objectiveCoefficients));
+                            getInvertedCoefficientSum(objectiveCoefficients));
         }
 
         // initialize the constraint rows
@@ -238,7 +219,7 @@ class SimplexTableau implements Serializable {
             // x-
             if (!restrictToNonNegative) {
                 matrix.setEntry(row, getSlackVariableOffset() - 1,
-                    getInvertedCoefficientSum(constraint.getCoefficients()));
+                                getInvertedCoefficientSum(constraint.getCoefficients()));
             }
 
             // RHS
@@ -253,7 +234,7 @@ class SimplexTableau implements Serializable {
 
             // artificial variables
             if ((constraint.getRelationship() == Relationship.EQ) ||
-                    (constraint.getRelationship() == Relationship.GEQ)) {
+                (constraint.getRelationship() == Relationship.GEQ)) {
                 matrix.setEntry(0, getArtificialVariableOffset() + artificialVar, 1);
                 matrix.setEntry(row, getArtificialVariableOffset() + artificialVar++, 1);
                 matrix.setRowVector(0, matrix.getRowVector(0).subtract(matrix.getRowVector(row)));
@@ -269,7 +250,7 @@ class SimplexTableau implements Serializable {
      * @return new versions of the constraints
      */
     public List<LinearConstraint> normalizeConstraints(Collection<LinearConstraint> originalConstraints) {
-        List<LinearConstraint> normalized = new ArrayList<LinearConstraint>();
+        List<LinearConstraint> normalized = new ArrayList<LinearConstraint>(originalConstraints.size());
         for (LinearConstraint constraint : originalConstraints) {
             normalized.add(normalize(constraint));
         }
@@ -333,6 +314,44 @@ class SimplexTableau implements Serializable {
      * @return the row that the variable is basic in.  null if the column is not basic
      */
     protected Integer getBasicRow(final int col) {
+        final int row = basicVariables[col];
+        return row == -1 ? null : row;
+    }
+
+    /**
+     * Returns the variable that is basic in this row.
+     * @param row the index of the row to check
+     * @return the variable that is basic for this row.
+     */
+    protected int getBasicVariable(final int row) {
+        return basicRows[row];
+    }
+
+    /**
+     * Initializes the basic variable / row mapping.
+     * @param startColumn the column to start
+     */
+    private void initializeBasicVariables(final int startColumn) {
+        basicVariables = new int[getWidth() - 1];
+        basicRows = new int[getHeight()];
+
+        Arrays.fill(basicVariables, -1);
+
+        for (int i = startColumn; i < getWidth() - 1; i++) {
+            Integer row = findBasicRow(i);
+            if (row != null) {
+                basicVariables[i] = row;
+                basicRows[row] = i;
+            }
+        }
+    }
+
+    /**
+     * Returns the row in which the given column is basic.
+     * @param col index of the column
+     * @return the row that the variable is basic in, or {@code null} if the variable is not basic.
+     */
+    private Integer findBasicRow(final int col) {
         Integer row = null;
         for (int i = 0; i < getHeight(); i++) {
             final double entry = getEntry(i, col);
@@ -354,12 +373,12 @@ class SimplexTableau implements Serializable {
             return;
         }
 
-        Set<Integer> columnsToDrop = new TreeSet<Integer>();
+        final Set<Integer> columnsToDrop = new TreeSet<Integer>();
         columnsToDrop.add(0);
 
         // positive cost non-artificial variables
         for (int i = getNumObjectiveFunctions(); i < getArtificialVariableOffset(); i++) {
-            final double entry = tableau.getEntry(0, i);
+            final double entry = getEntry(0, i);
             if (Precision.compareTo(entry, 0d, epsilon) > 0) {
                 columnsToDrop.add(i);
             }
@@ -373,12 +392,12 @@ class SimplexTableau implements Serializable {
             }
         }
 
-        double[][] matrix = new double[getHeight() - 1][getWidth() - columnsToDrop.size()];
+        final double[][] matrix = new double[getHeight() - 1][getWidth() - columnsToDrop.size()];
         for (int i = 1; i < getHeight(); i++) {
             int col = 0;
             for (int j = 0; j < getWidth(); j++) {
                 if (!columnsToDrop.contains(j)) {
-                    matrix[i - 1][col++] = tableau.getEntry(i, j);
+                    matrix[i - 1][col++] = getEntry(i, j);
                 }
             }
         }
@@ -391,6 +410,8 @@ class SimplexTableau implements Serializable {
 
         this.tableau = new Array2DRowRealMatrix(matrix);
         this.numArtificialVariables = 0;
+        // need to update the basic variable mappings as row/columns have been dropped
+        initializeBasicVariables(getNumObjectiveFunctions());
     }
 
     /**
@@ -406,8 +427,10 @@ class SimplexTableau implements Serializable {
      * @return whether the model has been solved
      */
     boolean isOptimal() {
-        for (int i = getNumObjectiveFunctions(); i < getWidth() - 1; i++) {
-            final double entry = tableau.getEntry(0, i);
+        final double[] objectiveFunctionRow = getRow(0);
+        final int end = getRhsOffset();
+        for (int i = getNumObjectiveFunctions(); i < end; i++) {
+            final double entry = objectiveFunctionRow[i];
             if (Precision.compareTo(entry, 0d, epsilon) < 0) {
                 return false;
             }
@@ -424,8 +447,8 @@ class SimplexTableau implements Serializable {
         Integer negativeVarBasicRow = negativeVarColumn > 0 ? getBasicRow(negativeVarColumn) : null;
         double mostNegative = negativeVarBasicRow == null ? 0 : getEntry(negativeVarBasicRow, getRhsOffset());
 
-        Set<Integer> basicRows = new HashSet<Integer>();
-        double[] coefficients = new double[getOriginalNumDecisionVariables()];
+        final Set<Integer> usedBasicRows = new HashSet<Integer>();
+        final double[] coefficients = new double[getOriginalNumDecisionVariables()];
         for (int i = 0; i < coefficients.length; i++) {
             int colIndex = columnLabels.indexOf("x" + i);
             if (colIndex < 0) {
@@ -438,12 +461,12 @@ class SimplexTableau implements Serializable {
                 // set the coefficient to 0 -> this case handles unconstrained
                 // variables that are still part of the objective function
                 coefficients[i] = 0;
-            } else if (basicRows.contains(basicRow)) {
+            } else if (usedBasicRows.contains(basicRow)) {
                 // if multiple variables can take a given value
                 // then we choose the first and set the rest equal to 0
                 coefficients[i] = 0 - (restrictToNonNegative ? 0 : mostNegative);
             } else {
-                basicRows.add(basicRow);
+                usedBasicRows.add(basicRow);
                 coefficients[i] =
                     (basicRow == null ? 0 : getEntry(basicRow, getRhsOffset())) -
                     (restrictToNonNegative ? 0 : mostNegative);
@@ -453,17 +476,46 @@ class SimplexTableau implements Serializable {
     }
 
     /**
-     * Subtracts a multiple of one row from another.
+     * Perform the row operations of the simplex algorithm with the selected
+     * pivot column and row.
+     * @param pivotCol the pivot column
+     * @param pivotRow the pivot row
+     */
+    protected void performRowOperations(int pivotCol, int pivotRow) {
+        // set the pivot element to 1
+        final double pivotVal = getEntry(pivotRow, pivotCol);
+        divideRow(pivotRow, pivotVal);
+
+        // set the rest of the pivot column to 0
+        for (int i = 0; i < getHeight(); i++) {
+            if (i != pivotRow) {
+                final double multiplier = getEntry(i, pivotCol);
+                if (multiplier != 0.0) {
+                    subtractRow(i, pivotRow, multiplier);
+                }
+            }
+        }
+
+        // update the basic variable mappings
+        final int previousBasicVariable = getBasicVariable(pivotRow);
+        basicVariables[previousBasicVariable] = -1;
+        basicVariables[pivotCol] = pivotRow;
+        basicRows[pivotRow] = pivotCol;
+    }
+
+    /**
+     * Divides one row by a given divisor.
      * <p>
      * After application of this operation, the following will hold:
-     * <pre>minuendRow = minuendRow - multiple * subtrahendRow</pre>
+     * <pre>dividendRow = dividendRow / divisor</pre>
      *
-     * @param dividendRow index of the row
+     * @param dividendRowIndex index of the row
      * @param divisor value of the divisor
      */
-    protected void divideRow(final int dividendRow, final double divisor) {
+    protected void divideRow(final int dividendRowIndex, final double divisor) {
+        final double[] dividendRow = getRow(dividendRowIndex);
         for (int j = 0; j < getWidth(); j++) {
-            tableau.setEntry(dividendRow, j, tableau.getEntry(dividendRow, j) / divisor);
+            dividendRow[j] /= divisor;
         }
     }
 
@@ -473,19 +525,15 @@ class SimplexTableau implements Serializable {
      * After application of this operation, the following will hold:
      * <pre>minuendRow = minuendRow - multiple * subtrahendRow</pre>
      *
-     * @param minuendRow row index
-     * @param subtrahendRow row index
-     * @param multiple multiplication factor
+     * @param minuendRowIndex row index
+     * @param subtrahendRowIndex row index
+     * @param multiplier multiplication factor
      */
-    protected void subtractRow(final int minuendRow, final int subtrahendRow,
-                               final double multiple) {
+    protected void subtractRow(final int minuendRowIndex, final int subtrahendRowIndex, final double multiplier) {
+        final double[] minuendRow = getRow(minuendRowIndex);
+        final double[] subtrahendRow = getRow(subtrahendRowIndex);
         for (int i = 0; i < getWidth(); i++) {
-            double result = tableau.getEntry(minuendRow, i) - tableau.getEntry(subtrahendRow, i) * multiple;
-            // cut-off values smaller than the cut-off threshold, otherwise may lead to numerical instabilities
-            if (FastMath.abs(result) < cutOff) {
-                result = 0.0;
-            }
-            tableau.setEntry(minuendRow, i, result);
+            minuendRow[i] -= subtrahendRow[i] * multiplier;
         }
     }
 
@@ -521,8 +569,7 @@ class SimplexTableau implements Serializable {
      * @param column column index
      * @param value for the entry
      */
-    protected final void setEntry(final int row, final int column,
-                                  final double value) {
+    protected final void setEntry(final int row, final int column, final double value) {
         tableau.setEntry(row, column, value);
     }
 
@@ -588,6 +635,15 @@ class SimplexTableau implements Serializable {
         return numArtificialVariables;
     }
 
+    /**
+     * Get the row from the tableau.
+     * @param row the row index
+     * @return the reference to the underlying row data
+     */
+    protected final double[] getRow(int row) {
+        return tableau.getDataRef()[row];
+    }
+
     /**
      * Get the tableau data.
      * @return tableau data
