@@ -18,10 +18,13 @@ package org.apache.commons.math3.stat;
 
 import java.io.Serializable;
 import java.text.NumberFormat;
+import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Iterator;
 import java.util.Comparator;
+import java.util.List;
 import java.util.Map;
+import java.util.Map.Entry;
 import java.util.TreeMap;
 
 import org.apache.commons.math3.exception.MathIllegalArgumentException;
@@ -39,16 +42,25 @@ import org.apache.commons.math3.util.MathUtils;
  * Integer values (int, long, Integer, Long) are not distinguished by type --
  * i.e. <code>addValue(Long.valueOf(2)), addValue(2), addValue(2l)</code> all have
  * the same effect (similarly for arguments to <code>getCount,</code> etc.).</p>
+ * <p>NOTE: byte and short values will be implicitly converted to int values
+ * by the compiler, thus there are no explicit overloaded methods for these
+ * primitive types.</p>
  * <p>
  * char values are converted by <code>addValue</code> to Character instances.
  * As such, these values are not comparable to integral values, so attempts
  * to combine integral types with chars in a frequency distribution will fail.
  * </p>
  * <p>
+ * Float is not coerced to Double.
+ * Since they are not Comparable with each other the user must do any necessary coercion.
+ * Float.NaN and Double.NaN are not treated specially; they may occur in input and will
+ * occur in output if appropriate.
+ * </b>
+ * <p>
  * The values are ordered using the default (natural order), unless a
  * <code>Comparator</code> is supplied in the constructor.</p>
  *
- * @version $Id: Frequency.java 1455703 2013-03-12 20:46:23Z tn $
+ * @version $Id: Frequency.java 1519820 2013-09-03 19:58:03Z tn $
  */
 public class Frequency implements Serializable {
 
@@ -76,8 +88,7 @@ public class Frequency implements Serializable {
     }
 
     /**
-     * Return a string representation of this frequency
-     * distribution.
+     * Return a string representation of this frequency distribution.
      *
      * @return a string representation.
      */
@@ -115,44 +126,12 @@ public class Frequency implements Serializable {
         incrementValue(v, 1);
     }
 
-    /**
-     * Increments the frequency count for v.
-     * <p>
-     * If other objects have already been added to this Frequency, v must
-     * be comparable to those that have already been added.
-     * </p>
-     *
-     * @param v the value to add.
-     * @param increment the amount by which the value should be incremented
-     * @throws IllegalArgumentException if <code>v</code> is not comparable with previous entries
-     * @since 3.1
-     */
-    public void incrementValue(Comparable<?> v, long increment){
-        Comparable<?> obj = v;
-        if (v instanceof Integer) {
-            obj = Long.valueOf(((Integer) v).longValue());
-        }
-        try {
-            Long count = freqTable.get(obj);
-            if (count == null) {
-                freqTable.put(obj, Long.valueOf(increment));
-            } else {
-                freqTable.put(obj, Long.valueOf(count.longValue() + increment));
-            }
-        } catch (ClassCastException ex) {
-            //TreeMap will throw ClassCastException if v is not comparable
-            throw new MathIllegalArgumentException(
-                  LocalizedFormats.INSTANCES_NOT_COMPARABLE_TO_EXISTING_VALUES,
-                  v.getClass().getName());
-        }
-    }
-
     /**
      * Adds 1 to the frequency count for v.
      *
      * @param v the value to add.
      * @throws MathIllegalArgumentException if the table contains entries not
-     * comparable to Integer
+     * comparable to Long
      */
     public void addValue(int v) throws MathIllegalArgumentException {
         addValue(Long.valueOf(v));
@@ -180,6 +159,89 @@ public class Frequency implements Serializable {
         addValue(Character.valueOf(v));
     }
 
+    /**
+     * Increments the frequency count for v.
+     * <p>
+     * If other objects have already been added to this Frequency, v must
+     * be comparable to those that have already been added.
+     * </p>
+     *
+     * @param v the value to add.
+     * @param increment the amount by which the value should be incremented
+     * @throws MathIllegalArgumentException if <code>v</code> is not comparable with previous entries
+     * @since 3.1
+     */
+    public void incrementValue(Comparable<?> v, long increment) throws MathIllegalArgumentException {
+        Comparable<?> obj = v;
+        if (v instanceof Integer) {
+            obj = Long.valueOf(((Integer) v).longValue());
+        }
+        try {
+            Long count = freqTable.get(obj);
+            if (count == null) {
+                freqTable.put(obj, Long.valueOf(increment));
+            } else {
+                freqTable.put(obj, Long.valueOf(count.longValue() + increment));
+            }
+        } catch (ClassCastException ex) {
+            //TreeMap will throw ClassCastException if v is not comparable
+            throw new MathIllegalArgumentException(
+                  LocalizedFormats.INSTANCES_NOT_COMPARABLE_TO_EXISTING_VALUES,
+                  v.getClass().getName());
+        }
+    }
+
+    /**
+     * Increments the frequency count for v.
+     * <p>
+     * If other objects have already been added to this Frequency, v must
+     * be comparable to those that have already been added.
+     * </p>
+     *
+     * @param v the value to add.
+     * @param increment the amount by which the value should be incremented
+     * @throws MathIllegalArgumentException if the table contains entries not
+     * comparable to Long
+     * @since 3.3
+     */
+    public void incrementValue(int v, long increment) throws MathIllegalArgumentException {
+        incrementValue(Long.valueOf(v), increment);
+    }
+
+    /**
+     * Increments the frequency count for v.
+     * <p>
+     * If other objects have already been added to this Frequency, v must
+     * be comparable to those that have already been added.
+     * </p>
+     *
+     * @param v the value to add.
+     * @param increment the amount by which the value should be incremented
+     * @throws MathIllegalArgumentException if the table contains entries not
+     * comparable to Long
+     * @since 3.3
+     */
+    public void incrementValue(long v, long increment) throws MathIllegalArgumentException {
+        incrementValue(Long.valueOf(v), increment);
+    }
+
+    /**
+     * Increments the frequency count for v.
+     * <p>
+     * If other objects have already been added to this Frequency, v must
+     * be comparable to those that have already been added.
+     * </p>
+     *
+     * @param v the value to add.
+     * @param increment the amount by which the value should be incremented
+     * @throws MathIllegalArgumentException if the table contains entries not
+     * comparable to Char
+     * @since 3.3
+     */
+    public void incrementValue(char v, long increment) throws MathIllegalArgumentException {
+        incrementValue(Character.valueOf(v), increment);
+    }
+
     /** Clears the frequency table */
     public void clear() {
         freqTable.clear();
@@ -232,7 +294,7 @@ public class Frequency implements Serializable {
     }
 
     /**
-     * Returns the number of values = v.
+     * Returns the number of values equal to v.
      * Returns 0 if the value is not comparable.
      *
      * @param v the value to lookup.
@@ -255,7 +317,7 @@ public class Frequency implements Serializable {
     }
 
     /**
-     * Returns the number of values = v.
+     * Returns the number of values equal to v.
      *
      * @param v the value to lookup.
      * @return the frequency of v.
@@ -265,7 +327,7 @@ public class Frequency implements Serializable {
     }
 
     /**
-     * Returns the number of values = v.
+     * Returns the number of values equal to v.
      *
      * @param v the value to lookup.
      * @return the frequency of v.
@@ -275,7 +337,7 @@ public class Frequency implements Serializable {
     }
 
     /**
-     * Returns the number of values = v.
+     * Returns the number of values equal to v.
      *
      * @param v the value to lookup.
      * @return the frequency of v.
@@ -298,7 +360,9 @@ public class Frequency implements Serializable {
      * Returns the percentage of values that are equal to v
      * (as a proportion between 0 and 1).
      * <p>
-     * Returns <code>Double.NaN</code> if no values have been added.</p>
+     * Returns <code>Double.NaN</code> if no values have been added.
+     * Returns 0 if at least one value has been added, but v is not comparable
+     * to the values set.</p>
      *
      * @param v the value to lookup
      * @return the proportion of values equal to v
@@ -493,6 +557,33 @@ public class Frequency implements Serializable {
         return getCumPct(Character.valueOf(v));
     }
 
+    /**
+     * Returns the mode value(s) in comparator order.
+     *
+     * @return a list containing the value(s) which appear most often.
+     * @since 3.3
+     */
+    public List<Comparable<?>> getMode() {
+        long mostPopular = 0; // frequencies are always positive
+
+        // Get the max count first, so we avoid having to recreate the List each time
+        for(Long l : freqTable.values()) {
+            long frequency = l.longValue();
+            if (frequency > mostPopular) {
+                mostPopular = frequency;
+            }
+        }
+
+        List<Comparable<?>> modeList = new ArrayList<Comparable<?>>();
+        for (Entry<Comparable<?>, Long> ent : freqTable.entrySet()) {
+            long frequency = ent.getValue().longValue();
+            if (frequency == mostPopular) {
+               modeList.add(ent.getKey());
+            }
+        }
+        return modeList;
+    }
+
     //----------------------------------------------------------------------------------------------
 
     /**
@@ -510,7 +601,7 @@ public class Frequency implements Serializable {
         final Iterator<Map.Entry<Comparable<?>, Long>> iter = other.entrySetIterator();
         while (iter.hasNext()) {
             final Map.Entry<Comparable<?>, Long> entry = iter.next();
-            incrementValue(entry.getKey(), entry.getValue());
+            incrementValue(entry.getKey(), entry.getValue().longValue());
         }
     }
 
