@@ -37,7 +37,7 @@ import org.apache.commons.math3.util.MathArrays;
  * <p>This method for multiple regression forms the solution to the OLS problem
  * by updating the QR decomposition as described by Gentleman.</p>
  *
- * @version $Id: MillerUpdatingRegression.java 1392358 2012-10-01 14:41:55Z psteitz $
+ * @version $Id: MillerUpdatingRegression.java 1547633 2013-12-03 23:03:06Z tn $
  * @since 3.0
  */
 public class MillerUpdatingRegression implements UpdatingMultipleLinearRegression {
@@ -342,14 +342,14 @@ public class MillerUpdatingRegression implements UpdatingMultipleLinearRegressio
         double total;
         final double eps = this.epsilon;
         for (int i = 0; i < nvars; i++) {
-            this.work_tolset[i] = Math.sqrt(d[i]);
+            this.work_tolset[i] = FastMath.sqrt(d[i]);
         }
         tol[0] = eps * this.work_tolset[0];
         for (int col = 1; col < nvars; col++) {
             pos = col - 1;
             total = work_tolset[col];
             for (int row = 0; row < col; row++) {
-                total += Math.abs(r[pos]) * work_tolset[row];
+                total += FastMath.abs(r[pos]) * work_tolset[row];
                 pos += nvars - row - 2;
             }
             tol[col] = eps * total;
@@ -383,7 +383,7 @@ public class MillerUpdatingRegression implements UpdatingMultipleLinearRegressio
         final double[] ret = new double[nreq];
         boolean rankProblem = false;
         for (int i = nreq - 1; i > -1; i--) {
-            if (Math.sqrt(d[i]) < tol[i]) {
+            if (FastMath.sqrt(d[i]) < tol[i]) {
                 ret[i] = 0.0;
                 d[i] = 0.0;
                 rankProblem = true;
@@ -413,7 +413,7 @@ public class MillerUpdatingRegression implements UpdatingMultipleLinearRegressio
     private void singcheck() {
         int pos;
         for (int i = 0; i < nvars; i++) {
-            work_sing[i] = Math.sqrt(d[i]);
+            work_sing[i] = FastMath.sqrt(d[i]);
         }
         for (int col = 0; col < nvars; col++) {
             // Set elements within R to zero if they are less than tol(col) in
@@ -422,7 +422,7 @@ public class MillerUpdatingRegression implements UpdatingMultipleLinearRegressio
             final double temp = tol[col];
             pos = col - 1;
             for (int row = 0; row < col - 1; row++) {
-                if (Math.abs(r[pos]) * work_sing[row] < temp) {
+                if (FastMath.abs(r[pos]) * work_sing[row] < temp) {
                     r[pos] = 0.0;
                 }
                 pos += nvars - row - 2;
@@ -625,7 +625,7 @@ public class MillerUpdatingRegression implements UpdatingMultipleLinearRegressio
         final int nvm = nvars - 1;
         final int base_pos = r.length - (nvm - in) * (nvm - in + 1) / 2;
         if (d[in] > 0.0) {
-            rms[in + rms_off] = 1.0 / Math.sqrt(d[in]);
+            rms[in + rms_off] = 1.0 / FastMath.sqrt(d[in]);
         }
         for (int col = in + 1; col < nvars; col++) {
             pos = base_pos + col - 1 - in;
@@ -635,7 +635,7 @@ public class MillerUpdatingRegression implements UpdatingMultipleLinearRegressio
                 pos += nvars - row - 2;
             }
             if (sumxx > 0.0) {
-                rms[col + rms_off] = 1.0 / Math.sqrt(sumxx);
+                rms[col + rms_off] = 1.0 / FastMath.sqrt(sumxx);
             } else {
                 rms[col + rms_off] = 0.0;
             }
@@ -645,7 +645,7 @@ public class MillerUpdatingRegression implements UpdatingMultipleLinearRegressio
             sumyy += d[row] * rhs[row] * rhs[row];
         }
         if (sumyy > 0.0) {
-            sumyy = 1.0 / Math.sqrt(sumyy);
+            sumyy = 1.0 / FastMath.sqrt(sumyy);
         }
         pos = 0;
         for (int col1 = in; col1 < nvars; col1++) {
@@ -729,10 +729,10 @@ public class MillerUpdatingRegression implements UpdatingMultipleLinearRegressio
             // Special cases.
             if (d1 > this.epsilon || d2 > this.epsilon) {
                 X = r[m1];
-                if (Math.abs(X) * Math.sqrt(d1) < tol[mp1]) {
+                if (FastMath.abs(X) * FastMath.sqrt(d1) < tol[mp1]) {
                     X = 0.0;
                 }
-                if (d1 < this.epsilon || Math.abs(X) < this.epsilon) {
+                if (d1 < this.epsilon || FastMath.abs(X) < this.epsilon) {
                     d[m] = d2;
                     d[mp1] = d1;
                     r[m1] = 0.0;
@@ -754,7 +754,7 @@ public class MillerUpdatingRegression implements UpdatingMultipleLinearRegressio
                     for (int _i = m1 + 1; _i < m1 + nvars - m - 1; _i++) {
                         r[_i] /= X;
                     }
-                    rhs[m] = rhs[m] / X;
+                    rhs[m] /= X;
                     bSkipTo40 = true;
                     //break;
                 }
@@ -868,7 +868,7 @@ public class MillerUpdatingRegression implements UpdatingMultipleLinearRegressio
         }
         double hii = 0.0;
         for (int col = 0; col < xrow.length; col++) {
-            if (Math.sqrt(d[col]) < tol[col]) {
+            if (FastMath.sqrt(d[col]) < tol[col]) {
                 wk[col] = 0.0;
             } else {
                 pos = col - 1;
