@@ -27,6 +27,7 @@ import org.apache.commons.math3.exception.DimensionMismatchException;
 import org.apache.commons.math3.exception.NumberIsTooLargeException;
 import org.apache.commons.math3.random.Well1024a;
 import org.apache.commons.math3.util.ArithmeticUtils;
+import org.apache.commons.math3.util.CombinatoricsUtils;
 import org.apache.commons.math3.util.FastMath;
 import org.junit.Assert;
 import org.junit.Test;
@@ -36,6 +37,7 @@ import org.junit.Test;
  */
 public class DerivativeStructureTest extends ExtendedFieldElementAbstractTest<DerivativeStructure> {
 
+    @Override
     protected DerivativeStructure build(final double x) {
         return new DerivativeStructure(2, 1, 0, x);
     }
@@ -87,6 +89,15 @@ public class DerivativeStructureTest extends ExtendedFieldElementAbstractTest<De
         }
     }
 
+    @Test
+    public void testCreateConstant() {
+        DerivativeStructure a = new DerivativeStructure(3, 2, 0, 1.3);
+        DerivativeStructure b = a.createConstant(2.5);
+        Assert.assertEquals(a.getFreeParameters(), b.getFreeParameters());
+        Assert.assertEquals(a.getOrder(), b.getOrder());
+        checkEquals(a.getField().getOne().multiply(2.5), b, 1.0e-15);
+    }
+
     @Test
     public void testPrimitiveAdd() {
         for (int maxOrder = 1; maxOrder < 5; ++maxOrder) {
@@ -174,7 +185,7 @@ public class DerivativeStructureTest extends ExtendedFieldElementAbstractTest<De
             DerivativeStructure r = new DerivativeStructure(1, 6, 0, x).reciprocal();
             Assert.assertEquals(1 / x, r.getValue(), 1.0e-15);
             for (int i = 1; i < r.getOrder(); ++i) {
-                double expected = ArithmeticUtils.pow(-1, i) * ArithmeticUtils.factorial(i) /
+                double expected = ArithmeticUtils.pow(-1, i) * CombinatoricsUtils.factorial(i) /
                                   FastMath.pow(x, i + 1);
                 Assert.assertEquals(expected, r.getPartialDerivative(i), 1.0e-15 * FastMath.abs(expected));
             }
@@ -214,6 +225,106 @@ public class DerivativeStructureTest extends ExtendedFieldElementAbstractTest<De
         }
     }
 
+    @Test
+    public void testPowDoubleDS() {
+        for (int maxOrder = 1; maxOrder < 5; ++maxOrder) {
+
+            DerivativeStructure x = new DerivativeStructure(3, maxOrder, 0, 0.1);
+            DerivativeStructure y = new DerivativeStructure(3, maxOrder, 1, 0.2);
+            DerivativeStructure z = new DerivativeStructure(3, maxOrder, 2, 0.3);
+            List<DerivativeStructure> list = Arrays.asList(x, y, z,
+                                                           x.add(y).add(z),
+                                                           x.multiply(y).multiply(z));
+
+            for (DerivativeStructure ds : list) {
+                // the special case a = 0 is included here
+                for (double a : new double[] { 0.0, 0.1, 1.0, 2.0, 5.0 }) {
+                    DerivativeStructure reference = (a == 0) ?
+                                                    x.getField().getZero() :
+                                                    new DerivativeStructure(3, maxOrder, a).pow(ds);
+                    DerivativeStructure result = DerivativeStructure.pow(a, ds);
+                    checkEquals(reference, result, 1.0e-15);
+                }
+
+            }
+
+            // negative base: -1^x can be evaluated for integers only, so value is sometimes OK, derivatives are always NaN
+            DerivativeStructure negEvenInteger = DerivativeStructure.pow(-2.0, new DerivativeStructure(3,  maxOrder, 0, 2.0));
+            Assert.assertEquals(4.0, negEvenInteger.getValue(), 1.0e-15);
+            Assert.assertTrue(Double.isNaN(negEvenInteger.getPartialDerivative(1, 0, 0)));
+            DerivativeStructure negOddInteger = DerivativeStructure.pow(-2.0, new DerivativeStructure(3,  maxOrder, 0, 3.0));
+            Assert.assertEquals(-8.0, negOddInteger.getValue(), 1.0e-15);
+            Assert.assertTrue(Double.isNaN(negOddInteger.getPartialDerivative(1, 0, 0)));
+            DerivativeStructure negNonInteger = DerivativeStructure.pow(-2.0, new DerivativeStructure(3,  maxOrder, 0, 2.001));
+            Assert.assertTrue(Double.isNaN(negNonInteger.getValue()));
+            Assert.assertTrue(Double.isNaN(negNonInteger.getPartialDerivative(1, 0, 0)));
+
+            DerivativeStructure zeroNeg = DerivativeStructure.pow(0.0, new DerivativeStructure(3,  maxOrder, 0, -1.0));
+            Assert.assertTrue(Double.isNaN(zeroNeg.getValue()));
+            Assert.assertTrue(Double.isNaN(zeroNeg.getPartialDerivative(1, 0, 0)));
+            DerivativeStructure posNeg = DerivativeStructure.pow(2.0, new DerivativeStructure(3,  maxOrder, 0, -2.0));
+            Assert.assertEquals(1.0 / 4.0, posNeg.getValue(), 1.0e-15);
+            Assert.assertEquals(FastMath.log(2.0) / 4.0, posNeg.getPartialDerivative(1, 0, 0), 1.0e-15);
+
+            // very special case: a = 0 and power = 0
+            DerivativeStructure zeroZero = DerivativeStructure.pow(0.0, new DerivativeStructure(3,  maxOrder, 0, 0.0));
+
+            // this should be OK for simple first derivative with one variable only ...
+            Assert.assertEquals(1.0, zeroZero.getValue(), 1.0e-15);
+            Assert.assertEquals(Double.NEGATIVE_INFINITY, zeroZero.getPartialDerivative(1, 0, 0), 1.0e-15);
+
+            // the following checks show a LIMITATION of the current implementation
+            // we have no way to tell x is a pure linear variable x = 0
+            // we only say: "x is a structure with value = 0.0,
+            // first derivative with respect to x = 1.0, and all other derivatives
+            // (first order with respect to y and z and higher derivatives) all 0.0.
+            // We have function f(x) = a^x and x = 0 so we compute:
+            // f(0) = 1, f'(0) = ln(a), f''(0) = ln(a)^2. The limit of these values
+            // when a converges to 0 implies all derivatives keep switching between
+            // +infinity and -infinity.
+            //
+            // Function composition rule for first derivatives is:
+            // d[f(g(x,y,z))]/dy = f'(g(x,y,z)) * dg(x,y,z)/dy
+            // so given that in our case x represents g and does not depend
+            // on y or z, we have dg(x,y,z)/dy = 0
+            // applying the composition rules gives:
+            // d[f(g(x,y,z))]/dy = f'(g(x,y,z)) * dg(x,y,z)/dy
+            //                 = -infinity * 0
+            //                 = NaN
+            // if we knew x is really the x variable and not the identity
+            // function applied to x, we would not have computed f'(g(x,y,z)) * dg(x,y,z)/dy
+            // and we would have found that the result was 0 and not NaN
+            Assert.assertTrue(Double.isNaN(zeroZero.getPartialDerivative(0, 1, 0)));
+            Assert.assertTrue(Double.isNaN(zeroZero.getPartialDerivative(0, 0, 1)));
+
+            // Function composition rule for second derivatives is:
+            // d2[f(g(x))]/dx2 = f''(g(x)) * [g'(x)]^2 + f'(g(x)) * g''(x)
+            // when function f is the a^x root and x = 0 we have:
+            // f(0) = 1, f'(0) = ln(a), f''(0) = ln(a)^2 which for a = 0 implies
+            // all derivatives keep switching between +infinity and -infinity
+            // so given that in our case x represents g, we have g(x) = 0,
+            // g'(x) = 1 and g''(x) = 0
+            // applying the composition rules gives:
+            // d2[f(g(x))]/dx2 = f''(g(x)) * [g'(x)]^2 + f'(g(x)) * g''(x)
+            //                 = +infinity * 1^2 + -infinity * 0
+            //                 = +infinity + NaN
+            //                 = NaN
+            // if we knew x is really the x variable and not the identity
+            // function applied to x, we would not have computed f'(g(x)) * g''(x)
+            // and we would have found that the result was +infinity and not NaN
+            if (maxOrder > 1) {
+                Assert.assertTrue(Double.isNaN(zeroZero.getPartialDerivative(2, 0, 0)));
+                Assert.assertTrue(Double.isNaN(zeroZero.getPartialDerivative(0, 2, 0)));
+                Assert.assertTrue(Double.isNaN(zeroZero.getPartialDerivative(0, 0, 2)));
+                Assert.assertTrue(Double.isNaN(zeroZero.getPartialDerivative(1, 1, 0)));
+                Assert.assertTrue(Double.isNaN(zeroZero.getPartialDerivative(0, 1, 1)));
+                Assert.assertTrue(Double.isNaN(zeroZero.getPartialDerivative(1, 1, 0)));
+            }
+
+        }
+
+    }
+
     @Test
     public void testExpression() {
         double epsilon = 2.5e-13;
@@ -610,6 +721,7 @@ public class DerivativeStructureTest extends ExtendedFieldElementAbstractTest<De
         }
     }
 
+    @Override
     @Test
     public void testExp() {
         double[] epsilon = new double[] { 1.0e-16, 1.0e-16, 1.0e-16, 1.0e-16, 1.0e-16 };
@@ -640,6 +752,7 @@ public class DerivativeStructureTest extends ExtendedFieldElementAbstractTest<De
         }
     }
 
+    @Override
     @Test
     public void testLog() {
         double[] epsilon = new double[] { 1.0e-16, 1.0e-16, 3.0e-14, 7.0e-13, 3.0e-11 };
@@ -648,7 +761,7 @@ public class DerivativeStructureTest extends ExtendedFieldElementAbstractTest<De
                 DerivativeStructure log = new DerivativeStructure(1, maxOrder, 0, x).log();
                 Assert.assertEquals(FastMath.log(x), log.getValue(), epsilon[0]);
                 for (int n = 1; n <= maxOrder; ++n) {
-                    double refDer = -ArithmeticUtils.factorial(n - 1) / FastMath.pow(-x, n);
+                    double refDer = -CombinatoricsUtils.factorial(n - 1) / FastMath.pow(-x, n);
                     Assert.assertEquals(refDer, log.getPartialDerivative(n), epsilon[n]);
                 }
             }
@@ -827,6 +940,7 @@ public class DerivativeStructureTest extends ExtendedFieldElementAbstractTest<De
         }
     }
 
+    @Override
     @Test
     public void testAtan2() {
         double[] epsilon = new double[] { 5.0e-16, 3.0e-15, 2.2e-14, 1.0e-12, 8.0e-11 };
@@ -1043,6 +1157,7 @@ public class DerivativeStructureTest extends ExtendedFieldElementAbstractTest<De
         }
     }
 
+    @Override
     @Test
     public void testAbs() {
 
@@ -1064,6 +1179,7 @@ public class DerivativeStructureTest extends ExtendedFieldElementAbstractTest<De
 
     }
 
+    @Override
     @Test
     public void testSignum() {
 
