@@ -69,7 +69,7 @@ import org.junit.Test;
  * See {@link NormalDistributionTest} and {@link ChiSquaredDistributionTest}
  * for examples.
  *
- * @version $Id: RealDistributionAbstractTest.java 1428822 2013-01-04 12:28:44Z erans $
+ * @version $Id: RealDistributionAbstractTest.java 1533974 2013-10-20 20:42:41Z psteitz $
  */
 public abstract class RealDistributionAbstractTest {
 
@@ -95,6 +95,9 @@ public abstract class RealDistributionAbstractTest {
     /** Values used to test density calculations */
     private double[] densityTestValues;
 
+    /** Values used to test logarithmic density calculations */
+    private double[] logDensityTestValues;
+
     //-------------------- Abstract methods -----------------------------------
 
     /** Creates the default continuous distribution instance to use in tests. */
@@ -109,6 +112,18 @@ public abstract class RealDistributionAbstractTest {
     /** Creates the default density test expected values */
     public abstract double[] makeDensityTestValues();
 
+    /** Creates the default logarithmic density test expected values.
+     * The default implementation simply computes the logarithm
+     * of each value returned by {@link #makeDensityTestValues()}.*/
+    public double[] makeLogDensityTestValues() {
+        final double[] densityTestValues = makeDensityTestValues();
+        final double[] logDensityTestValues = new double[densityTestValues.length];
+        for (int i = 0; i < densityTestValues.length; i++) {
+            logDensityTestValues[i] = FastMath.log(densityTestValues[i]);
+        }
+        return logDensityTestValues;
+    }
+
     //---- Default implementations of inverse test data generation methods ----
 
     /** Creates the default inverse cumulative probability test input values */
@@ -134,6 +149,7 @@ public abstract class RealDistributionAbstractTest {
         inverseCumulativeTestPoints = makeInverseCumulativeTestPoints();
         inverseCumulativeTestValues = makeInverseCumulativeTestValues();
         densityTestValues = makeDensityTestValues();
+        logDensityTestValues = makeLogDensityTestValues();
     }
 
     /**
@@ -147,6 +163,7 @@ public abstract class RealDistributionAbstractTest {
         inverseCumulativeTestPoints = null;
         inverseCumulativeTestValues = null;
         densityTestValues = null;
+        logDensityTestValues = null;
     }
 
     //-------------------- Verification methods -------------------------------
@@ -208,6 +225,19 @@ public abstract class RealDistributionAbstractTest {
         }
     }
 
+    /**
+     * Verifies that logarithmic density calculations match expected values
+     */
+    protected void verifyLogDensities() {
+        // FIXME: when logProbability methods are added to RealDistribution in 4.0, remove cast below
+        for (int i = 0; i < cumulativeTestPoints.length; i++) {
+            TestUtils.assertEquals("Incorrect probability density value returned for "
+                    + cumulativeTestPoints[i], logDensityTestValues[i],
+                    ((AbstractRealDistribution) distribution).logDensity(cumulativeTestPoints[i]),
+                    getTolerance());
+        }
+    }
+
     //------------------------ Default test cases -----------------------------
 
     /**
@@ -237,6 +267,15 @@ public abstract class RealDistributionAbstractTest {
         verifyDensities();
     }
 
+    /**
+     * Verifies that logarithmic density calculations return expected values
+     * for default test instance data
+     */
+    @Test
+    public void testLogDensities() {
+        verifyLogDensities();
+    }
+
     /**
      * Verifies that probability computations are consistent
      */
