@@ -17,6 +17,7 @@
 package org.apache.commons.math3.linear;
 
 import java.math.BigDecimal;
+
 import org.apache.commons.math3.TestUtils;
 import org.apache.commons.math3.fraction.BigFraction;
 import org.apache.commons.math3.fraction.Fraction;
@@ -31,12 +32,14 @@ import org.junit.Test;
 /**
  * Test cases for the {@link MatrixUtils} class.
  *
- * @version $Id: MatrixUtilsTest.java 1403587 2012-10-30 00:21:20Z erans $
+ * @version $Id: MatrixUtilsTest.java 1533638 2013-10-18 21:19:18Z tn $
  */
 
 public final class MatrixUtilsTest {
 
     protected double[][] testData = { {1d,2d,3d}, {2d,5d,3d}, {1d,0d,8d} };
+    protected double[][] testData3x3Singular = { { 1, 4, 7, }, { 2, 5, 8, }, { 3, 6, 9, } };
+    protected double[][] testData3x4 = { { 12, -51, 4, 1 }, { 6, 167, -68, 2 }, { -4, 24, -41, 3 } };
     protected double[][] nullMatrix = null;
     protected double[] row = {1,2,3};
     protected BigDecimal[] bigRow =
@@ -364,6 +367,18 @@ public final class MatrixUtilsTest {
         }
     }
 
+    @Test(expected=SingularMatrixException.class)
+    public void testBlockInverseNonInvertible() {
+        final double[][] data = {
+            { -1, 0, 123, 4 },
+            { -56, 78.9, -0.1, -23.4 },
+            { 5.67, 8, -9, 1011 },
+            { 5.67, 8, -9, 1011 },
+        };
+
+        MatrixUtils.blockInverse(new Array2DRowRealMatrix(data), 2);
+    }
+
     @Test
     public void testIsSymmetric() {
         final double eps = Math.ulp(1d);
@@ -433,4 +448,38 @@ public final class MatrixUtilsTest {
         };
         MatrixUtils.checkSymmetric(MatrixUtils.createRealMatrix(dataNonSym), Math.ulp(1d));
     }
+    
+    @Test(expected=SingularMatrixException.class)
+    public void testInverseSingular() {
+        RealMatrix m = MatrixUtils.createRealMatrix(testData3x3Singular);
+        MatrixUtils.inverse(m);
+    }
+    
+    @Test(expected=NonSquareMatrixException.class)
+    public void testInverseNonSquare() {
+        RealMatrix m = MatrixUtils.createRealMatrix(testData3x4);
+        MatrixUtils.inverse(m);
+    }
+    
+    @Test
+    public void testInverseDiagonalMatrix() {
+        final double[] data = { 1, 2, 3 };
+        final RealMatrix m = new DiagonalMatrix(data);
+        final RealMatrix inverse = MatrixUtils.inverse(m);
+
+        final RealMatrix result = m.multiply(inverse);
+        TestUtils.assertEquals("MatrixUtils.inverse() returns wrong result",
+                MatrixUtils.createRealIdentityMatrix(data.length), result, Math.ulp(1d));
+    }
+
+    @Test
+    public void testInverseRealMatrix() {
+        RealMatrix m = MatrixUtils.createRealMatrix(testData);
+        final RealMatrix inverse = MatrixUtils.inverse(m);
+
+        final RealMatrix result = m.multiply(inverse);
+        TestUtils.assertEquals("MatrixUtils.inverse() returns wrong result",
+                MatrixUtils.createRealIdentityMatrix(testData.length), result, 1e-12);
+    }
+
 }
