@@ -32,7 +32,7 @@ import org.apache.commons.math3.exception.MathIllegalArgumentException;
 /**
  * Test cases for the {@link SparseFieldMatrix} class.
  *
- * @version $Id: SparseFieldMatrixTest.java 1364030 2012-07-21 01:10:04Z erans $
+ * @version $Id: SparseFieldMatrixTest.java 1569825 2014-02-19 17:19:59Z luc $
  */
 public class SparseFieldMatrixTest {
     // 3 x 3 identity matrix
@@ -265,7 +265,7 @@ public class SparseFieldMatrixTest {
         assertClose("identity operate", testVector, m.operate(testVector),
                 entryTolerance);
         assertClose("identity operate", testVector, m.operate(
-                new ArrayFieldVector<Fraction>(testVector)).getData(), entryTolerance);
+                new ArrayFieldVector<Fraction>(testVector)).toArray(), entryTolerance);
         m = createSparseMatrix(bigSingular);
         try {
             m.operate(testVector);
