@@ -17,7 +17,6 @@
 
 package org.apache.commons.math3.optim.nonlinear.vector.jacobian;
 
-import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.List;
 import org.apache.commons.math3.optim.PointVectorValuePair;
@@ -30,7 +29,6 @@ import org.apache.commons.math3.optim.nonlinear.vector.ModelFunction;
 import org.apache.commons.math3.optim.nonlinear.vector.ModelFunctionJacobian;
 import org.apache.commons.math3.analysis.MultivariateVectorFunction;
 import org.apache.commons.math3.analysis.MultivariateMatrixFunction;
-import org.apache.commons.math3.exception.ConvergenceException;
 import org.apache.commons.math3.exception.DimensionMismatchException;
 import org.apache.commons.math3.exception.TooManyEvaluationsException;
 import org.apache.commons.math3.exception.MathUnsupportedOperationException;
@@ -40,7 +38,6 @@ import org.apache.commons.math3.util.FastMath;
 import org.apache.commons.math3.util.Precision;
 import org.junit.Assert;
 import org.junit.Test;
-import org.junit.Ignore;
 
 /**
  * <p>Some of the unit tests are re-implementations of the MINPACK <a
@@ -381,8 +378,8 @@ public class LevenbergMarquardtOptimizerTest
                         for (int i = 0; i < values.length; ++i) {
                             final double t = time.get(i);
                             values[i] = params[0] +
-                                params[1] * Math.exp(-t / params[3]) +
-                                params[2] * Math.exp(-t / params[4]);
+                                params[1] * FastMath.exp(-t / params[3]) +
+                                params[2] * FastMath.exp(-t / params[4]);
                         }
                         return values;
                     }
@@ -402,10 +399,10 @@ public class LevenbergMarquardtOptimizerTest
                             final double p4 =  params[4];
                             final double tOp3 = t / p3;
                             final double tOp4 = t / p4;
-                            jacobian[i][1] = Math.exp(-tOp3);
-                            jacobian[i][2] = Math.exp(-tOp4);
-                            jacobian[i][3] = params[1] * Math.exp(-tOp3) * tOp3 / p3;
-                            jacobian[i][4] = params[2] * Math.exp(-tOp4) * tOp4 / p4;
+                            jacobian[i][1] = FastMath.exp(-tOp3);
+                            jacobian[i][2] = FastMath.exp(-tOp4);
+                            jacobian[i][3] = params[1] * FastMath.exp(-tOp3) * tOp3 / p3;
+                            jacobian[i][4] = params[2] * FastMath.exp(-tOp4) * tOp4 / p4;
                         }
                         return jacobian;
                     }
