@@ -17,7 +17,6 @@
 
 package org.apache.commons.math3.optim.nonlinear.vector.jacobian;
 
-import java.io.Serializable;
 import java.util.Arrays;
 import org.apache.commons.math3.exception.TooManyEvaluationsException;
 import org.apache.commons.math3.analysis.MultivariateVectorFunction;
@@ -610,8 +609,6 @@ public class MinpackTest {
     }
 
     private static class LinearFullRankFunction extends MinpackFunction {
-        private static final long serialVersionUID = -9030323226268039536L;
-        
         public LinearFullRankFunction(int m, int n, double x0,
                                       double theoreticalStartCost,
                                       double theoreticalMinCost) {
@@ -649,8 +646,6 @@ public class MinpackTest {
     }
 
     private static class LinearRank1Function extends MinpackFunction {
-        private static final long serialVersionUID = 8494863245104608300L;
-
         public LinearRank1Function(int m, int n, double x0,
                                    double theoreticalStartCost,
                                    double theoreticalMinCost) {
@@ -684,8 +679,6 @@ public class MinpackTest {
     }
 
     private static class LinearRank1ZeroColsAndRowsFunction extends MinpackFunction {
-        private static final long serialVersionUID = -3316653043091995018L;
-
         public LinearRank1ZeroColsAndRowsFunction(int m, int n, double x0) {
             super(m, buildArray(n, x0),
                   FastMath.sqrt((m * (m + 3) - 6) / (2.0 * (2 * m - 3))),
@@ -728,7 +721,6 @@ public class MinpackTest {
     }
 
     private static class RosenbrockFunction extends MinpackFunction {
-        private static final long serialVersionUID = 2893438180956569134L;
         public RosenbrockFunction(double[] startParams, double theoreticalStartCost) {
             super(2, startParams, 0.0, buildArray(2, 1.0));
         }
@@ -748,7 +740,6 @@ public class MinpackTest {
     }
 
     private static class HelicalValleyFunction extends MinpackFunction {
-        private static final long serialVersionUID = 220613787843200102L;
         public HelicalValleyFunction(double[] startParams,
                                      double theoreticalStartCost) {
             super(3, startParams, 0.0, new double[] { 1.0, 0.0, 0.0 });
@@ -794,8 +785,6 @@ public class MinpackTest {
     }
 
     private static class PowellSingularFunction extends MinpackFunction {
-        private static final long serialVersionUID = 7298364171208142405L;
-
         public PowellSingularFunction(double[] startParams,
                                       double theoreticalStartCost) {
             super(4, startParams, 0.0, buildArray(4, 0.0));
@@ -834,8 +823,6 @@ public class MinpackTest {
   }
 
     private static class FreudensteinRothFunction extends MinpackFunction {
-        private static final long serialVersionUID = 2892404999344244214L;
-
         public FreudensteinRothFunction(double[] startParams,
                                         double theoreticalStartCost,
                                         double theoreticalMinCost,
@@ -865,8 +852,6 @@ public class MinpackTest {
     }
 
     private static class BardFunction extends MinpackFunction {
-        private static final long serialVersionUID = 5990442612572087668L;
-
         public BardFunction(double x0,
                             double theoreticalStartCost,
                             double theoreticalMinCost,
@@ -914,8 +899,6 @@ public class MinpackTest {
     }
 
     private static class KowalikOsborneFunction extends MinpackFunction {
-        private static final long serialVersionUID = -4867445739880495801L;
-
         public KowalikOsborneFunction(double[] startParams,
                                       double theoreticalStartCost,
                                       double theoreticalMinCost,
@@ -970,8 +953,6 @@ public class MinpackTest {
     }
 
     private static class MeyerFunction extends MinpackFunction {
-        private static final long serialVersionUID = -838060619150131027L;
-
         public MeyerFunction(double[] startParams,
                              double theoreticalStartCost,
                              double theoreticalMinCost,
@@ -1021,8 +1002,6 @@ public class MinpackTest {
     }
 
     private static class WatsonFunction extends MinpackFunction {
-        private static final long serialVersionUID = -9034759294980218927L;
-
         public WatsonFunction(int n, double x0,
                               double theoreticalStartCost,
                               double theoreticalMinCost,
@@ -1092,8 +1071,6 @@ public class MinpackTest {
     }
 
     private static class Box3DimensionalFunction extends MinpackFunction {
-        private static final long serialVersionUID = 5511403858142574493L;
-
         public Box3DimensionalFunction(int m, double[] startParams,
                                        double theoreticalStartCost) {
             super(m, startParams, 0.0,
@@ -1132,8 +1109,6 @@ public class MinpackTest {
     }
 
     private static class JennrichSampsonFunction extends MinpackFunction {
-        private static final long serialVersionUID = -2489165190443352947L;
-
         public JennrichSampsonFunction(int m, double[] startParams,
                                        double theoreticalStartCost,
                                        double theoreticalMinCost,
@@ -1168,8 +1143,6 @@ public class MinpackTest {
     }
 
     private static class BrownDennisFunction extends MinpackFunction {
-        private static final long serialVersionUID = 8340018645694243910L;
-
         public BrownDennisFunction(int m, double[] startParams,
                                    double theoreticalStartCost,
                                    double theoreticalMinCost,
@@ -1216,8 +1189,6 @@ public class MinpackTest {
     }
 
     private static class ChebyquadFunction extends MinpackFunction {
-        private static final long serialVersionUID = -2394877275028008594L;
-
         private static double[] buildChebyquadArray(int n, double factor) {
             double[] array = new double[n];
             double inv = factor / (n + 1);
@@ -1294,8 +1265,6 @@ public class MinpackTest {
     }
 
     private static class BrownAlmostLinearFunction extends MinpackFunction {
-        private static final long serialVersionUID = 8239594490466964725L;
-
         public BrownAlmostLinearFunction(int m, double factor,
                                          double theoreticalStartCost,
                                          double theoreticalMinCost,
@@ -1355,8 +1324,6 @@ public class MinpackTest {
     }
 
     private static class Osborne1Function extends MinpackFunction {
-        private static final long serialVersionUID = 4006743521149849494L;
-
         public Osborne1Function(double[] startParams,
                                 double theoreticalStartCost,
                                 double theoreticalMinCost,
@@ -1408,8 +1375,6 @@ public class MinpackTest {
     }
 
     private static class Osborne2Function extends MinpackFunction {
-        private static final long serialVersionUID = -8418268780389858746L;
-
         public Osborne2Function(double[] startParams,
                                 double theoreticalStartCost,
                                 double theoreticalMinCost,
