@@ -43,7 +43,6 @@ import org.apache.commons.math3.distribution.WeibullDistribution;
 import org.apache.commons.math3.distribution.ZipfDistribution;
 import org.apache.commons.math3.distribution.ZipfDistributionTest;
 import org.apache.commons.math3.stat.Frequency;
-import org.apache.commons.math3.stat.descriptive.SummaryStatistics;
 import org.apache.commons.math3.stat.inference.ChiSquareTest;
 import org.apache.commons.math3.util.FastMath;
 import org.apache.commons.math3.exception.MathIllegalArgumentException;
@@ -54,7 +53,7 @@ import org.junit.runner.RunWith;
 /**
  * Test cases for the RandomDataGenerator class.
  *
- * @version $Id: RandomDataGeneratorTest.java 1457491 2013-03-17 17:15:31Z psteitz $
+ * @version $Id: RandomDataGeneratorTest.java 1547633 2013-12-03 23:03:06Z tn $
  */
 @RunWith(RetryRunner.class)
 public class RandomDataGeneratorTest {
@@ -421,9 +420,8 @@ public class RandomDataGeneratorTest {
             checkNextPoissonConsistency(randomData.nextUniform(1, 1000));
         }
         // large means
-        // TODO: When MATH-282 is resolved, s/3000/10000 below
         for (int i = 1; i < 10; i++) {
-            checkNextPoissonConsistency(randomData.nextUniform(1000, 3000));
+            checkNextPoissonConsistency(randomData.nextUniform(1000, 10000));
         }
     }
 
@@ -822,8 +820,7 @@ public class RandomDataGeneratorTest {
         randomData.reSeed(1000);
         double v = randomData.nextUniform(0, 1);
         randomData.reSeed();
-        Assert.assertTrue("different seeds", Math
-                .abs(v - randomData.nextUniform(0, 1)) > 10E-12);
+        Assert.assertTrue("different seeds", FastMath.abs(v - randomData.nextUniform(0, 1)) > 10E-12);
         randomData.reSeed(1000);
         Assert.assertEquals("same seeds", v, randomData.nextUniform(0, 1), 10E-12);
         randomData.reSeedSecure(1000);
@@ -957,7 +954,7 @@ public class RandomDataGeneratorTest {
         }
         
         String[] labels = {"{0, 1, 2}", "{ 0, 2, 1 }", "{ 1, 0, 2 }",
-        		"{ 1, 2, 0 }", "{ 2, 0, 1 }", "{ 2, 1, 0 }"};
+                "{ 1, 2, 0 }", "{ 2, 0, 1 }", "{ 2, 1, 0 }"};
         TestUtils.assertChiSquareAccept(labels, expected, observed, 0.001);
 
         // Check size = 1 boundary case
