@@ -28,7 +28,7 @@ import org.junit.Assert;
 import org.junit.Test;
 
 /**
- * @version $Id: BetaTest.java 1415853 2012-11-30 21:02:23Z celestin $
+ * @version $Id: BetaTest.java 1546350 2013-11-28 11:41:12Z erans $
  */
 public class BetaTest {
 
@@ -134,6 +134,26 @@ public class BetaTest {
         testRegularizedBeta(0.75, 0.5, 1.0, 2.0);
     }
 
+    @Test
+    public void testRegularizedBetaTinyArgument() {
+        double actual = Beta.regularizedBeta(1e-17, 1.0, 1e12);
+        // This value is from R: pbeta(1e-17,1,1e12)
+        TestUtils.assertEquals(9.999950000166648e-6, actual, 1e-16);
+    }
+
+    @Test
+    public void testMath1067() {
+        final double x = 0.22580645161290325;
+        final double a = 64.33333333333334;
+        final double b = 223;
+
+        try {
+            final double r = Beta.regularizedBeta(x, a, b, 1e-14, 10000);
+        } catch (StackOverflowError error) {
+            Assert.fail("Infinite recursion");
+        }
+    }
+
     @Test
     public void testLogBetaNanPositive() {
         testLogBeta(Double.NaN, Double.NaN, 2.0);
@@ -170,7 +190,7 @@ public class BetaTest {
     }
 
     /**
-     * Reference data for the {@link Gamma#logGammaSum(double, double)}
+     * Reference data for the {@link #logGammaSum(double, double)}
      * function. This data was generated with the following
      * <a href="http://maxima.sourceforge.net/">Maxima</a> script.
      *
